require 'rubygems'
require 'RMagick'

class Image
  attr_accessor :data
  attr_accessor :filename
  attr_accessor :width
  attr_accessor :height

  def initialize(filename, data = [])
    @data = data
    @filename = filename
  end

  def self.load(filenames, width, height)
    filenames.map do |filename|
      view = Magick::Image.read(filename).first.view(0, 0, width, height)
      img = Image.new(filename)
      img.width = width
      img.height = height
      view.height.times do |i|
        view.width.times do |j|
          img.data << (black?(view[i][j]) ? 1 : 0 )
        end
      end
      img
    end
  end

  def add_noise(level)
    resolution = @width * @height
    pool = (0...resolution).to_a.shuffle.first(resolution * level / 10)
    img = Magick::Image.read(filename).first
    view = img.view(0, 0, @width, @height)
    pool.each do |val|
      x, y = val.divmod(@width)
      view[x][y] = invert(view[x][y])
    end
    view.sync
    img
  end

  def self.white_pixel
    Magick::Pixel.new(65535, 65535, 65535)
  end

  def self.black_pixel
    Magick::Pixel.new(0, 0, 0)
  end

  def <=> (other)
    self.filename <=> other.filename
  end

  private

  def invert(pixel)
    if Image.black?(pixel)
      Image.white_pixel
    else
      Image.black_pixel
    end
  end

  def self.black?(pixel)
    pixel.red.zero? && pixel.green.zero? && pixel.blue.zero?
  end
end
