class Neuron
  attr_accessor :id
  attr_accessor :weights
  attr_accessor :network
  attr_accessor :wins

  def initialize(id, network, weights = nil)
    @id = id
    @network = network
    @weights = weights || Array.new(network.number_of_synapses){rand}
    @wins = {}
    log_state('initial state', true)
  end

  def increase_link_with(input)
    @weights = @weights.add(input.data.sub(@weights).mul(@network.speed))
    @wins[input.filename] ||= 0
    @wins[input.filename] += 1
    log_state("win on #{input.filename}")
  end

  def reaction_on(input)
    wins = @wins.inject(0){|s,v| s+=v.last}
    wins = 1 if wins.zero?
    input.data.sub(@weights).dist * wins
  end

  protected
  def log_state(message = nil, truncate = false)
    File.open("neuron-#{id}.txt", truncate ? "w" : "a") do |f|
      f.puts(message) if message
      f.puts(@weights.inspect)
    end
  end
end
